

set -ex



$PREFIX/bin/llvm-config --libs
$PREFIX/bin/llc -version
test -f $PREFIX/include/llvm/Pass.h
test -f $PREFIX/lib/libLLVMSupport.a
test -f $PREFIX/lib/libLLVMCore.a
$PREFIX/bin/opt -dot-cfg cfg_test.ll
python test_cfg_dot.py
opt -S -vector-library=SVML -mcpu=haswell -O3 numba-3016.ll -o out.ll
grep __svml_sin4 out.ll
exit 0
